from .test_calendar import *
from .test_event import *
from .test_feed import *
from .test_forms import *
from .test_occurrence import *
from .test_periods import *
from .test_perms import *
from .test_recurrent_event import *
from .test_rule import *
from .test_templatetags import *
from .test_utils import *
from .test_views import *
