{% load scheduletags staticfiles %}
<div class="prevnext">
    <a href="{% prev_url target slug period %}">
      <img align="top" border="0" src="{% static "schedule/img/left_mod.png" %}"/>
    </a> 
    &nbsp; <b>{{period_name}}</b> &nbsp; 
    <a href="{% next_url target slug period %}">
      <img align="top" border="0" src="{% static "schedule/img/right_mod.png" %}"/>
    </a>
</div>

