{% load scheduletags %}
  <div class="slot_column" style="width:{{width_slot}}px;height:{{height}}px;">
    {% for slot in slots %}
    <div class="slot" style="top:{{slot.top}}px;height:{{slot.height}}px;width:{{width}}px;">
      <span class="time">{{ slot.start|time:"G:i" }}</span>
      {% if addable %}
      {% create_event_url calendar slot.start %}
      {% endif %}
    </div>
    {% endfor %}
  </div>
  <div class="occ_column" style="left:{{width_slot}}px;width:{{width_occ}}px;height:{{height}}px;">
  {% for occ in occurrences %}
      <div href="#{% hash_occurrence occ %}" class="occ type{{occ.cls}}{% if occ.cancelled %} cancelled{% endif %}" 
      style="top:{{occ.top}}px;left:{{occ.left}}px;width:{{occ.width}}px;height:{{occ.height}}px;" onclick="openDetail(this);">
        {% options occ %}
        {% title occ %}

      </div>
      <div id="{% hash_occurrence occ %}" style="display:none;">
         {% include 'schedule/_detail.html' with occurrence=occ %}
      </div>
  {% endfor %}
  </div>
