{% extends "schedule/base.html" %}
{% load i18n %}

{% block body %}
<h1>{% trans "Delete" %}</h1>
{% trans "Are you sure that you really want to cancel this occurrence?" %}

<form action="." method="POST">
  {% csrf_token %}
  <h2>{{ occurrence.title }}</h2>
  <strong>From</strong> {{ occurrence.start|date:_("DATETIME_FORMAT") }}
  <strong>to</strong> {{ occurrence.end|date:_("DATETIME_FORMAT") }} <br/>
  <input type="submit" value="{% trans "Delete" %}" name="submit" />
  <input type="button" value="{% trans "Cancel" %}" onclick="window.location='{{next}}'"/>
  <input type="hidden" name="next" value="{{next|default:""}}"/>
</form>
{% endblock %}
